import simpleobserver
from .fields import Compound

__all__ = [
    'Schema'
]


class Schema(object):

    def __init__(self, *mixins, **fields):
        self.mixins = mixins
        self.fields = {}

        for m in self.mixins:
            self.fields.update(m.fields)
        self.fields.update(fields)

        self.validator = Compound(**self.fields)
        
        events = ['validate']
        self.before = simpleobserver.Subject(*events)
        self.after = simpleobserver.Subject(*events)


    def validate(self, fields):
        for m in self.mixins:
            m.before.fire('validate', fields)
        self.before.fire('validate', fields)

        validated_fields = self.validator.validate(fields)

        for m in self.mixins:
            m.after.fire('validate', validated_fields)
        self.after.fire('validate', validated_fields)

        return validated_fields
        


__all__ = ['strtodatetime', 'parsedate']

try:
    import timelib
    strtodatetime = timelib.strtodatetime
except ImportError:
    strtodatetime = None

try:
    from dateutil import parser as date_parser
    parsedate = date_parser.parse
except ImportError:
    parsedate = None


import io
import os.path
from setuptools import setup, find_packages

setup(
	name='valipede',
	version="1.0.1",
	description='Data validation',
	long_description=io.open('README.rst', mode='r', encoding='utf-8').read(),
	classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries'
    ],
    keywords='validation',
    author='Elisha Fitch-Cook',
    author_email='elisha@cooper.com',
    url='http://github.com/cooper-software/valipede',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'six',
        'simpleobserver'
    ],
    tests_require=[
        'mock',
        'timelib', 
        'python-dateutil'
    ],
    test_suite='test_valipede'
)